/* 
 * Copyright (c) 2001, 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: makevalue.c,v 1.2 2002/02/25 11:13:16 malekith Exp $
 *
 * Runtime enviroment for gont.
 *
 * Assumes compiler that allows $ in identifiers, such as GCC.
 * (It is required to have Ksi, so this ain't so important)
 */

#include <gont/runtime.h>

#include <string.h>
#include <stdarg.h>

/* Make new lambda expression. */
struct gont_lambda_exp *gont_make_lambda(void *fnc, void *closure)
{
	struct gont_lambda_exp *p;
	
	p = gont_xmalloc(sizeof(*p));
	
	p->fnc = fnc;
	p->closure = closure;
	
	return p;
}

/* Copy given float, so that it can be returned from function. */
double *gont_make_float(double v)
{
	double *ptr = gont_xmalloc_atomic(sizeof(*ptr));
	
	*ptr = v;
	
	return ptr;
}

/* Contruct new string. */
struct gont_string *gont_make_string(ptroff size, const char *content)
{
	struct gont_string *r;

	r = gont_xmalloc_atomic(sizeof(*r) + size);
	r->len = size;
	r->data = (char*)r + sizeof(*r);
	memcpy(r->data, content, size);

	return r;
}

struct gont_union *gont_make_union(ptroff field_no, void *value)
{
	struct gont_union *r;

	r = gont_xmalloc(sizeof(struct gont_union));
	r->field_no = field_no;
	r->value = value;

	return r;
}

void **gont_make_tuple(ptroff n, ...)
{
	va_list ap;
	void **p;
	int i;
	
	p = gont_xmalloc(n * sizeof(void*));
	
	va_start(ap, n);
	for (i = 0; i < n; i++) 
		p[i] = va_arg(ap, void*);
	va_end(ap);
	
	return p;
}
