/* 
 * Copyright (c) 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: std.g,v 1.5 2002/02/26 13:05:27 malekith Exp $
 *
 * Runtime environment for Gont, Std module implementation.
 */


/* Just wrappers... */
extern_c void Std__print_string(string);
extern_c void Std__prerr_string(string);
extern_c string Std__read_line();
extern_c string Std__itoa(int);
extern_c string Std__ftoa(float);
extern_c int Std__atoi(string);
extern_c float Std__atof(string);
extern_c void Std__exit(int);
extern_c void Std__at_exit(*() -> void);
extern_c bool gont_streq(string, string);
extern_c int gont_strcmp(string, string);
extern_c string gont_strjoin(string, string);

string itoa(int i) { return Std__itoa(i); }
string ftoa(float f) { return Std__ftoa(f); }
int atoi(string s) { return Std__atoi(s); }
float atof(string s) { return Std__atof(s); }
void print_string(string s) { Std__print_string(s); }
void prerr_string(string s) { Std__prerr_string(s); }
void at_exit(*() -> void f) { Std__at_exit(f); }
bool streq(string a, string b) { return gont_streq(a, b); }
int strcmp(string a, string b) { return gont_strcmp(a, b); }
string strjoin(string a, string b) { return gont_strjoin(a, b); }

/* ... and very simple functions... */
'a exit(int r)
{
	Std__exit(r); 
	raise Exit; 	// never reached
}

/* Stdout output. */
void print_int(int i)
{
	print_string(itoa(i)); 
}

void print_float(float f)
{
	print_string(ftoa(f)); 
}

void print_newline()
{
	print_string("\n");
}

void print_endline(string s)
{
	print_string(s);
	print_newline();
}

/* Stderr output. */
void prerr_int(int i)
{
	prerr_string(itoa(i)); 
}

void prerr_float(float f)
{
	prerr_string(ftoa(f)); 
}

void prerr_newline()
{
	prerr_string("\n");
}

void prerr_endline(string s)
{
	prerr_string(s);
	prerr_newline();
}

/* Input. */
string read_line()
{
	return Std__read_line();
}

int read_int()
{
	return atoi(read_line());
}

float read_float()
{
	return atof(read_line());
}

/* Raise shortcuts. */
'a fail_with(string s)
{
	raise Failure[s];
}

'a invalid_argument(string s)
{
	raise Invalid_argument[s];
}

/* Pair opertations. */
'a fst(*['a, 'b] p)
{
	switch p {
	case [a, _]: return a;
	}

	raise Match_failure;	// never reached
}

'b snd(*['a, 'b] p)
{
	switch p {
	case [_, a]: return a;
	}

	raise Match_failure;	// never reached
}

void __dummy_disaster()
{
	// make sure these are linked (other are referenced elsewhere in
	// this module)
	raise Null_access;
	raise Not_found;
	raise End_of_file;
}
