/* 
 * Copyright (c) 2002 The University of Wroclaw.
 * All rights reserved. See file COPYRIGHT for details about licensing.
 *
 * $Id: stdc.c,v 1.4 2002/02/26 13:05:04 malekith Exp $
 *
 * C implementation for Std module.
 */

#include <gont/runtime.h>

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

extern ptroff Std$$Match_failure;
extern ptroff Std$$Null_access;
extern ptroff Std$$End_of_file;
extern void Std$$invalid_argument(struct gont_string *);

static struct gont_string *gont_cstring(const char *s)
{
	return gont_make_string(strlen(s), s);
}

void Std__print_string(struct gont_string *s)
{
	fwrite(s->data, s->len, 1, stdout);
}

void Std__prerr_string(struct gont_string *s)
{
	fwrite(s->data, s->len, 1, stderr);
}

struct gont_string *Std__read_line_f(FILE *f)
{
	char buf[4096];
	char *r;

	r = fgets(buf, sizeof(buf), f);

	if (r == NULL)
		gont_raise(gont_make_union(Std$$End_of_file, NULL));
	
	return gont_cstring(buf);
}

struct gont_string *Std__read_line()
{
	return Std__read_line_f(stdin);
}

struct gont_string *Std__itoa(int i)
{
	char buf[100];

	sprintf(buf, "%d", i);
	
	return gont_cstring(buf);
}

struct gont_string *Std__ftoa(double *f)
{
	char buf[100];

	sprintf(buf, "%g", *f);
	
	return gont_cstring(buf);
}

void gont_match_failure()
{
	gont_raise(gont_make_union(Std$$Match_failure, NULL));
}

void gont_null_access()
{
	gont_raise(gont_make_union(Std$$Null_access, NULL));
}

ptroff Std__atoi(struct gont_string *s)
{
	char buf[s->len + 1];
	char *p;
	ptroff r;

	if (s->len == 0)
		goto fail;
		
	memcpy(buf, s->data, s->len);
	buf[s->len] = 0;

	r = strtol(buf, &p, 0);

	while (isspace(*p))
		p++;
		
	if (*p != '\0')
		goto fail;
	
	return r;

fail:
	Std$$invalid_argument(gont_cstring("Std::atoi"));
	return -1;	/* never reached */
}

double *Std__atof(struct gont_string *s)
{
	char buf[s->len + 1];
	char *p;
	double r;

	if (s->len == 0)
		goto fail;
		
	memcpy(buf, s->data, s->len);
	buf[s->len] = 0;

	r = strtod(buf, &p);

	while (isspace(*p))
		p++;
		
	if (*p != '\0')
		goto fail;
	
	return gont_make_float(r);

fail:
	Std$$invalid_argument(gont_cstring("Std::atof"));
	return NULL;	/* never reached */
}

void Std__exit(ptroff x)
{
	gont_run_exit_routines();
	exit(x);
}

void Std__at_exit(struct gont_lambda_exp *fnc)
{
	gont_register_exit_routine((void (*)(void*))fnc->fnc, fnc->closure);
}

ptroff gont_streq(struct gont_string *s1, struct gont_string *s2)
{
	return s1->len == s2->len &&
	       memcmp(s1->data, s2->data, s1->len) == 0;
}

ptroff gont_strcmp(struct gont_string *s1, struct gont_string *s2)
{
	int r;
	
	if (s1->len < s2->len) {
		r = memcmp(s1->data, s2->data, s1->len);
		return r == 0 ? -1 : r;
	} else if (s1->len > s2->len) {
		r = memcmp(s1->data, s2->data, s2->len);
		return r == 0 ? 1 : r;
	} else
		return memcmp(s1->data, s2->data, s1->len);
}

struct gont_string *gont_strjoin(struct gont_string *s1, 
			         struct gont_string *s2)
{
	struct gont_string *r;

	r = gont_xmalloc_atomic(sizeof(*r) + s1->len + s2->len);
	r->len = s1->len + s2->len;
	r->data = (char*)r + sizeof(*r);
	memcpy(r->data, s1->data, s1->len);
	memcpy(r->data + s1->len, s2->data, s2->len);

	return r;
}
